unit IDEPlugin;

interface

uses
  Borland.Studio.ToolsAPI,
  Commons.Settings,
  Options,
  System.Collections,
  System.ComponentModel,
  System.Drawing,
  System.Resources,
  System.Windows.Forms;

type
  TIDEPlugin = class(System.ComponentModel.Component)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    components: System.ComponentModel.IContainer;
    ImageList: System.Windows.Forms.ImageList;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    procedure Dispose(Disposing: Boolean); override;
  strict private type
    TShutdownWizard = class(System.Object, IOTAWizard)
    strict private
      WizardIndex: Integer;
      Service: IOTAWizardService;
      FIDEShutdown: EventHandler;
    public
      function get_IDString: string;
      function get_Name: string;
    public
      property IDString: string read get_IDString;
      property Name: string read get_Name;
      property IDEShutdown: EventHandler add FIDEShutdown remove FIDEShutdown;
      procedure Execute;
      procedure Destroyed;
      constructor Create;
      destructor Destroy; override;
    end;
  strict private
    ShutdownWizard: TShutdownWizard;
    OTAService: IOTAService;
    MainMenuService: IOTAMainMenuService;
    MenuItemAdd,MenuItemRemove: IOTAMenuItem;
    Options: TOptions;
  strict private
    procedure IDEShutdownHandler(Sender: TObject; Args: EventArgs);
    procedure FileNotificationHandler(Sender: TObject; Args: FileNotificationEventArgs);
    procedure AdjustSelection(Add: Boolean; Text: string);
    procedure OnMenuExecutedAdd(Sender: TObject; Args: EventArgs);
    procedure OnMenuExecutedRemove(Sender: TObject; Args: EventArgs);
  public
    class procedure IDERegister; static;
    constructor Create;
  end;

  [assembly: RuntimeRequiredAttribute(TypeOf(TIDEPlugin))]

implementation

uses
  &Message,
  System.IO,
  System.Text;

{ TIDEPlugin.TShutdownWizard }

constructor TIDEPlugin.TShutdownWizard.Create;
begin
  inherited Create;
  Service := BorlandIDE.GetService(TypeOf(IOTAWizardService)) as IOTAWizardService;
  WizardIndex := Service.AddWizard(Self);
end;

destructor TIDEPlugin.TShutdownWizard.Destroy;
begin
  Service.RemoveWizard(WizardIndex);
  inherited Destroy;
end;

procedure TIDEPlugin.TShutdownWizard.Destroyed;
begin
  if Assigned(FIDEShutdown) then
    FIDEShutdown(Self,EventArgs.Create);
end;

procedure TIDEPlugin.TShutdownWizard.Execute;
begin
end;

function TIDEPlugin.TShutdownWizard.get_IDString: string;
begin
  Result := 'TIDEPlugin.Regionalize.TShutdownWizard';
end;

function TIDEPlugin.TShutdownWizard.get_Name: string;
begin
  Result := 'IDEPlugin Regionalize ShutdownWizard';
end;

{ TIDEPlugin }

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TIDEPlugin.InitializeComponent;
var
  resources: System.Resources.ResourceManager;
begin
  Self.components := System.ComponentModel.Container.Create;
  resources := System.Resources.ResourceManager.Create(TypeOf(TIDEPlugin));
  Self.ImageList := System.Windows.Forms.ImageList.Create(Self.components);
  //
  // ImageList
  //
  Self.ImageList.ImageSize := System.Drawing.Size.Create(16, 16);
  Self.ImageList.ImageStream := (System.Windows.Forms.ImageListStreamer(resources.GetObject('I' +
    'mageList.ImageStream')));
  Self.ImageList.TransparentColor := System.Drawing.Color.Transparent;
end;
{$ENDREGION}

class procedure TIDEPlugin.IDERegister;
begin
  TIDEPlugin.Create;
end;

procedure TIDEPlugin.Dispose(Disposing: Boolean);
begin
  if Disposing then begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

constructor TIDEPlugin.Create;
begin
  inherited Create;
  InitializeComponent;
  Options := TOptions.Load;
  MainMenuService := BorlandIDE.GetService(typeof(IOTAMainMenuService)) as IOTAMainMenuService;
  MainMenuService.AddMenuItem('HTMLTidySubmenuItem',OTAMenuItemLocation.otamlAfter,'RegionalizeMenuItem','&Regionalize');
  MenuItemRemove := MainMenuService.AddMenuItem('RegionalizeMenuItem',OTAMenuItemLocation.otamlChild,
                                                'RegionalizeRemoveSubmenuItem','&Remove Region around Selection');
  MenuItemAdd := MainMenuService.AddMenuItem('RegionalizeMenuItem',OTAMenuItemLocation.otamlChild,
                                             'RegionalizeAddSubmenuItem','&Make Region of Selection',BitMap(ImageList.Images.Item[0]).GetHBitMap);
  Include(MenuItemAdd.Executed, OnMenuExecutedAdd);
  Include(MenuItemRemove.Executed, OnMenuExecutedRemove);
  ShutdownWizard := TIDEPlugin.TShutdownWizard.Create;
  OTAService := BorlandIDE.GetService(typeof(IOTAService)) as IOTAService;
  Include(OTAService.FileNotification,FileNotificationHandler);
  Include(ShutdownWizard.IDEShutdown,IDEShutdownHandler);
end;

procedure TIDEPlugin.IDEShutdownHandler(Sender: TObject; Args: EventArgs);
begin
  Exclude(OTAService.FileNotification,FileNotificationHandler);
  MainMenuService.RemoveMenuItem('RegionalizeAddSubmenuItem');
  MainMenuService.RemoveMenuItem('RegionalizeRemoveSubmenuItem');
  MainMenuService.RemoveMenuItem('RegionalizeMenuItem');
  ShutdownWizard.Free;
end;

procedure TIDEPlugin.FileNotificationHandler(Sender: TObject; Args: FileNotificationEventArgs);

  function ConverShortCut(ShortCut: Keys): Integer;
  const
    KeyShift = $2000;
    KeyControl = $4000;
    KeyAlt = $8000;
  begin
    Result := 0;
    if (Keys.Control and ShortCut) <> Keys.None then
      Result := Result or KeyControl;
    if (Keys.Shift and ShortCut) <> Keys.None then
      Result := Result or KeyShift;
    if (Keys.Alt and ShortCut) <> Keys.None then
      Result := Result or KeyAlt;
    ShortCut := ShortCut and Keys.KeyCode;
    Result := Result or Convert.ToInt32(ShortCut);
  end;

begin
  if ((Args.NotifyCode = OTAFileNotification.ofnPackageInstalled) or
			(Args.NotifyCode = OTAFileNotification.ofnPackageUninstalled)  or
			(Args.NotifyCode = OTAFileNotification.ofnFileOpened)) then begin
    if Options.ItemRemoveShortcut <> Keys.None then
      MenuItemRemove.ShortCut := ConverShortCut(Options.ItemRemoveShortcut);
    if Options.ItemAddShortcut <> Keys.None then
      MenuItemAdd.ShortCut := ConverShortCut(Options.ItemAddShortcut);
  end;
end;

procedure TIDEPlugin.AdjustSelection(Add: Boolean; Text: string);
var
  ModuleServices: IOTAModuleServices;
  Module: IOTAModule;
  Editor: IOTAEditor;
  SourceEditor: IOTASourceEditor;
  View: IOTAEditView;
  Reader: IOTAFileReader;
  Writer: IOTAFileWriter;
  BlockStart: OTACharPos;
  BlockAfter: OTACharPos;
  StartPos, EndPos: LongInt;
  I: Integer;
  S: string;

  procedure AdjustSelectedText(var SelectedText: string);
  var
    Lines: ArrayList;
    Reader: StringReader;
    SB: &StringBuilder;
    I: Integer;
    S: string;
		Titled: Boolean;
  begin
    Lines := ArrayList.Create;
    Reader := StringReader.Create(SelectedText);
    S := Reader.ReadLine;
    while Assigned(S) do begin
      Lines.Add(S);
      S := Reader.ReadLine;
    end;
    Reader.Close;
    SB := &StringBuilder.Create;
    S := string(Lines[0]).Trim;
		Titled := (Lines.Count > 0) and (Length(S) >= 2) and (S.Substring(0,2) = '//');
    if Add and ((not Titled and (Lines.Count > 0) and (string(Lines[0]).ToLower.IndexOf('{$region') < 0)) or
                (Titled and (Lines.Count > 1) and (string(Lines[1]).ToLower.IndexOf('{$region') < 0))) then begin
      if Titled then begin
        SB := SB.Append('{$region ''' + S.Substring(2).Trim + '''}');
        SB.Append(&Environment.NewLine);
      end
      else begin
        SB := SB.Append('{$region}');
        SB.Append(&Environment.NewLine);
        SB.Append(Lines[0]);
        SB.Append(&Environment.NewLine);
      end;
      for I := 1 to Pred(Lines.Count) do begin
        SB.Append(Lines[I]);
        SB.Append(&Environment.NewLine);
      end;
      SB.Append('{$endregion}');
      SB.Append(&Environment.NewLine);
      SelectedText := SB.ToString;
    end
    else if not Add and (Lines.Count > 1) and
            (string(Lines[0]).ToLower.IndexOf('{$region') >= 0) and
            (string(Lines[Pred(Lines.Count)]).ToLower.IndexOf('{$endregion') >= 0) then begin
      for I := 1 to Pred(Lines.Count) - 1 do begin
        SB.Append(Lines[I]);
        SB.Append(&Environment.NewLine);
      end;
      SelectedText := SB.ToString;
    end;
  end;

begin
  ModuleServices := BorlandIDE.GetService(typeof(IOTAModuleServices)) as IOTAModuleServices;
  Module := ModuleServices.CurrentModule;
  if not Assigned(Module) then
    Exit;
  for I := 0 to Pred(Module.ModuleFileCount) do begin
    Editor := Module.ModuleFileEditors(I);
    if Editor is IOTASourceEditor then begin
      SourceEditor := IOTASourceEditor(Editor);
      Break;
    end;
  end;
  if not Assigned(SourceEditor) then
    Exit;
  if SourceEditor.EditViewCount > 1 then begin
    with TFormMessage.Create do begin
      &Message := 'Too many views open, please close all but one';
      ShowDialog;
      Free;
    end;
    (SourceEditor as IOTAEditor).Show;
    Exit;
  end;
  if SourceEditor.EditViewCount < 1 then
    Exit;
  View := SourceEditor.GetEditView(0);
  if SourceEditor.BlockType = OTABlockType.btColumn then
    Exit;
  try
    BlockStart := SourceEditor.BlockStart;
    BlockAfter := SourceEditor.BlockAfter;
    BlockStart.CharIndex := 0;
    if BlockAfter.CharIndex > 0 then begin
      BlockAfter.CharIndex := 0;
      Inc(BlockAfter.Line);
    end;
    StartPos := View.CharPosToPos(BlockStart);
    EndPos := View.CharPosToPos(BlockAfter);
    if (EndPos - StartPos - 1) > 0 then begin
      Reader := SourceEditor.CreateReader;
      Reader.Read(StartPos,0);
      S := System.Text.Encoding.UTF8.GetString(Reader.Read(EndPos - StartPos - 1,0));
      Reader.Close;
      AdjustSelectedText(S);
      Writer := SourceEditor.CreateWriter;
      Writer.CopyTo(StartPos);
      Writer.DeleteTo(EndPos);
      Writer.Insert(S);
      Writer.Close;
      (SourceEditor as IOTAEditor).MarkModified;
    end;
  except
    ;
  end;
  (SourceEditor as IOTAEditor).Show;
end;

procedure TIDEPlugin.OnMenuExecutedAdd(Sender: TObject; Args: EventArgs);
begin
  AdjustSelection(true,'');
end;

procedure TIDEPlugin.OnMenuExecutedRemove(Sender: TObject; Args: EventArgs);
begin
  AdjustSelection(false,'');
end;


end.
